import greenfoot.*;
import java.awt.Color;
import java.util.ArrayList;

public class LinearCounter  extends Actor
{
    private int cap;
    private int current;
  
    ArrayList<GreenfootImage> onMarkers;
    ArrayList<GreenfootImage> offMarkers;
    private int markerWidth;
    private int markerHeight;
    
    boolean squares;
  
    private String label;
  
    /**
    * Construct the Counter using default images (squares)
    * including a label
    *
    */
    public LinearCounter(int cap, int startValue, String label)
    {
        this.cap = cap;
        setCurrent(startValue);
        this.label = label; 
    
        createSquares();
        squares = true;
        
        createImage();
    }
  
    /**
     * Construct the Counter using default images (squares)
     *
     */
    public LinearCounter(int cap, int startValue)
    {
        this(cap, startValue, null);
    }  
  
    /**
     * Construct the Counter, using a pair of given images, one for 'off' and one for 'on', including a label.
     * 
     * @param onMarker The iamge used if the unit is 'on'
     * @param offMarker the image used for 'off' units -  'on' and 'off' must be the same size
     *
     */
    public LinearCounter(GreenfootImage onMarker, GreenfootImage offMarker, int cap, int startValue, String label)
    {
        this.cap = cap;
        setCurrent(startValue);
        this.label = label;
    
        onMarkers = new ArrayList<GreenfootImage>();
        offMarkers = new ArrayList<GreenfootImage>();        
        
        for (int i=0; i<cap; i++) {
            onMarkers.add(onMarker);
            offMarkers.add(offMarker);
        }
        
        squares = false;
    
        createImage();
    }
  
    /**
     * Construct the Counter, using a pair of given images, one for 'off' and one for 'on'.
     * 
     * @param onMarker The iamge used if the unit is 'on'
     * @param offMarker the image used for 'off' units -  'on' and 'off' must be the same size
     *
     */
    public LinearCounter(GreenfootImage onMarker, GreenfootImage offMarker, int cap, int startValue)
    {
        this(onMarker, offMarker, cap, startValue, null);
    }  
  
    private void createImage()
    {
        // Scale the markers so that they match size, as the bigger of the two
        // in each dimension.
        int biggerX = Math.max(onMarkers.get(0).getWidth(), offMarkers.get(0).getWidth());
        int biggerY = Math.max(onMarkers.get(0).getHeight(), offMarkers.get(0).getHeight());
        
        markerWidth = biggerX;
        markerHeight = biggerY;
    
        for (int i=0; i<cap; i++) {
            onMarkers.get(i).scale(biggerX, biggerY);
            offMarkers.get(i).scale(biggerX, biggerY);
        }
        
        /*
         * set the height of the image to the height of the markers, and the 
         * width to the marker width * the number of markers + the width of
         * the label, if there is one.
         */
        int labelWidth = 0;
        if (label != null) {
            labelWidth = label.length() * 10;
        }
    
        int imgWidth = (markerWidth +2) * cap + 2 + labelWidth;
        int imgHeight = markerHeight + 2;
    
        setImage(new GreenfootImage(imgWidth, imgHeight));
    
        updateDisplay();
    }
  
    private void updateDisplay()
    {
        GreenfootImage img = getImage();
    
        img.clear();
    
        // draw onImgs on the display up to the current value
        for (int i=0; i<current; i++) {
            GreenfootImage marker = onMarkers.get(i);
            img.drawImage(marker, (markerWidth+2)*i, 0);
        }
    
        // draw offImgs on the display up to the cap
        for (int i=current; i<cap; i++) {
            GreenfootImage marker = offMarkers.get(i);
            img.drawImage(marker, (markerWidth+2)*i, 0);
        }
    
        // draw the label, if necessary
        if (label != null) {
            img.setColor(Color.white);
            img.drawString(label, (markerWidth+2)*cap +2, markerHeight);
        }
      
        setImage(img);
    }
    
    /**
     * resets the image for current values - useful if they have been changed
     */
    public void updateImage()
    {
        if (squares) {
            createSquares();
        }
        else {
            for (int i=0; i<cap; i++) {
                onMarkers.add(onMarkers.get(0));
                offMarkers.add(offMarkers.get(0));
            }
        }
    
        createImage();
    }
    
    private void createSquares()
    {
        onMarkers = new ArrayList<GreenfootImage>();
        offMarkers = new ArrayList<GreenfootImage>();
    
        int colourGrade = 255 / cap;
    
        for (int i=0; i<cap; i++) {
            GreenfootImage om = new GreenfootImage(10,10);
            GreenfootImage fm = new GreenfootImage(10,10);
            
            om.setColor(new Color(255, 255 - colourGrade*i, 0));
            fm.setColor(new Color(255, 255 - colourGrade*i, 0));
            
            om.fill();
            fm.drawRect(0,0,9,9);
            
            onMarkers.add(om);
            offMarkers.add(fm);
        }
    }
  
    /**
     * add 1 to the current total tracked by the Counter
     *
     * @returns true unless the total was already as high as the Counter's cap
     */
    public boolean addCount()
    {
        if (current == cap) {
            return false;
        }
        else {
            current ++;
        }
    
        updateDisplay();
        return true;
    }
  
    /**
     * subtract 1 from the current total tracked by the Counter
     *
     * @returns true unless the total was already 0
     */
    public boolean subtractCount()
    {
        if (current == 0) {
            return false;
        }
        else {
            current --;
        }
    
        updateDisplay();
        return true;
    }
  
    public void setCap(int value)
    {
        cap = value;
        updateImage();
    }
    public int getCap()
    {
        return cap;
    }
    public void setCurrent(int value)
    {
        current = value;
    }
    public int getCurrent()
    {
        return current;
    }
    public void setLabel(String text)
    {
        label = text;
        updateDisplay();
    }
}      
    