import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)

/**
 * A variation of an actor that maintains precise location (using doubles for the co-ordinates
 * instead of ints). It also maintains a current movement in form of a movement vector.
 * 
 * This is a variation of the SmoothMover class presented ealier in the book (version 2.0).
 * This version implements wrap-around movement: when the actor moves out of the world at one
 * side, it enters it again at the opposite edge.
 * 
 * Addition: commented out is an implementation for spinning, as well.
 * 
 * @author Poul Henriksen
 * @author Michael Kolling
 * 
 * @version 2.1
 */
public abstract class SmoothMover extends Actor
{
    private Vector movement;
    private double exactX;
    private double exactY;
    private double spin;
    private boolean wrap;
    
    public SmoothMover()
    {
        this(new Vector());
    }
    
    /**
     * Create new thing initialised with given speed.
     */
    public SmoothMover(Vector movement)
    {
        this.movement = movement;
        wrap = true;
    }
    
    /**
     * as above, but with spin
     */
    public SmoothMover(Vector movement, double spin)
    {
        this.movement = movement;
        this.spin = spin;
        wrap = true;
    }
    
    /**
     * Move in the current movement direction. Wrap around to the opposite edge of the
     * screen if moving out of the world.
     */
    public void move() 
    {
        exactX = exactX + movement.getX();
        exactY = exactY + movement.getY();
        if (wrap) {
            if(exactX >= getWorld().getWidth()) {
            exactX = 0;
            }
            if(exactX < 0) {
                exactX = getWorld().getWidth() - 1;
            }
            if(exactY >= getWorld().getHeight()) {
                exactY = 0;
            }
            if(exactY < 0) {
                exactY = getWorld().getHeight() - 1;
            }
        }
        super.setLocation((int) exactX, (int) exactY);
        spin();
        if (!wrap && (exactX >= getWorld().getWidth()  ||
                exactX < 0                       ||
                exactY >= getWorld().getHeight() ||
                exactY < 0 )) 
        {
            getWorld().removeObject(this);
        }
    }
    
    /**
     * Set the location from exact coordinates.
     */
    public void setLocation(double x, double y) 
    {
        exactX = x;
        exactY = y;
        super.setLocation((int) x, (int) y);
    }
    
    /**
     * Set the location from int coordinates.
     */
    public void setLocation(int x, int y) 
    {
        exactX = x;
        exactY = y;
        super.setLocation(x, y);
    }

    /**
     * Return the exact x-coordinate (as a double).
     */
    public double getExactX() 
    {
        return exactX;
    }

    /**
     * Return the exact y-coordinate (as a double).
     */
    public double getExactY() 
    {
        return exactY;
    }

    /**
     * Increase the speed with the given vector.
     */
    public void addForce(Vector force) 
    {
        movement.add(force);
    }
    
    /**
     * Accelerate the speed of this mover by the given factor. (Factors < 1 will
     * decelerate.)
     */
    public void accelerate(double factor)
    {
        movement.scale(factor);
        if (movement.getLength() < 0.15) {
            movement.setNeutral();
        }
    }
    
    /**
     * Return the speed of this actor.
     */
    public double getSpeed()
    {
        return movement.getLength();
    }
    
    /**
     * Stop movement of this actor.
     */
    public void stop()
    {
        movement.setNeutral();
    }
    
    /**
     * Return the current speed.
     */
    public Vector getMovement() 
    {
        return movement;
    }
    
    public double getSpin()
    {
        return spin;
    }
    
    public void setSpin(double factor)
    {
        spin = factor;
    }
    
    public void addSpin(double factor)
    {
        spin += factor;
    }
    
    private void spin()
    {
        Double spin = this.spin;
        setRotation(getRotation() + spin.intValue());
    }
    
    public void setWrap(boolean tof)
    {
        wrap = tof;
    }
}
